# coding:utf-8
from collections import defaultdict

from educommon.utils.db import get_related_instances
from m3.actions import OperationResult
from objectpack.actions import ObjectDeleteAction
from objectpack.models import ModelProxy

from .mixins import CascadeDeleteMixin
from .ui import RelatedObjectsWindow


class DeleteCheck(object):

    u"""Проверяет объекты перед удалением на наличие зависимостей.

    В случае, если в системе есть объекты, зависимые от удаляемого объекта, то
    отменяет удаление и выводит окно с предупреждением и списком зависимых
    объектов.
    """

    def _get_objects(self, context):
        model = self.action.parent.model
        if issubclass(model, ModelProxy):
            model = model.model

        object_ids = getattr(context, self.action.parent.id_param_name)

        return model.objects.filter(pk__in=object_ids)

    def _get_related_instances(self, obj, skip_func):
        if isinstance(obj, ModelProxy):
            obj = getattr(obj, obj.model.__name__.lower())
        return get_related_instances(obj, collapse=False, skip_func=skip_func)

    def before(self, request, context):
        if not isinstance(self.action, ObjectDeleteAction):
            return

        # Удаляемые объекты
        objects = self._get_objects(context)

        if not objects:
            return

        if hasattr(self.action.parent, 'skip_field'):
            def skip_function(field):
                return (
                    self.action.parent.skip_field(field) or
                    CascadeDeleteMixin.skip_field(field)
                )
        else:
            skip_function = CascadeDeleteMixin.skip_field

        # Объекты, зависящие от удаляемых
        related_objects = defaultdict(list)
        for obj in objects:
            for related_obj in self._get_related_instances(obj, skip_function):
                key = '.'.join((
                    related_obj._meta.app_label,
                    related_obj._meta.object_name,
                ))
                related_objects[key].append(related_obj)

        if not related_objects:
            return

        title = (
            u'Удаление {} невозможно, т.к. на {} есть ссылки:'
            .format(
                u'объекта' if len(objects) == 1 else u'объектов',
                u'него' if len(objects) == 1 else u'них'
            )
        )
        win = RelatedObjectsWindow()
        win.set_params(dict(
            title=title,
            objects=objects,
            related_objects=related_objects,
        ))
        return OperationResult(
            success=False,
            code=win.get_script(),
        )
