# coding: utf-8
from .base import ModelDataSource


class DataSourceRegistry(object):

    u"""Реестр параметров данных.

    При первом чтении из реестра отправляет сигнал ``init`` для добавления
    в реестр источников данных системы. Каждое django-приложение должно
    в обработчике этого сигнала зарегистрировать свои источники данных.
    """

    def __init__(self):
        u"""Инициализация реестра.

        По завершении инициализации отправляет сигнал ``post_init``.
        """
        self._data_sources = {}

    def register(self, data_source):
        u"""Регистрация источника данных."""
        assert isinstance(data_source, ModelDataSource)
        assert data_source.name
        assert data_source.name not in self._data_sources

        self._data_sources[data_source.name] = data_source

    def get(self, data_source_name):
        u"""Возвращает источник данных по имени.

        :param str data_source_name: Имя источника данных.
        """
        return self._data_sources[data_source_name]

    def __contains__(self, key):
        return key in self._data_sources

    def iterkeys(self):
        return self._data_sources.iterkeys()

    def itervalues(self):
        return self._data_sources.itervalues()

    def iteritems(self):
        return self._data_sources.iteritems()


registry = DataSourceRegistry()
